/* -----------------------------------------------------------------------------
*
*  npcsv.c
*
*  ANTz - realtime 3D data visualization tools for the real-world, based on NPE.
*
*  ANTz is hosted at http://openantz.com and NPE at http://neuralphysics.org
*
*  Written in 2010-2013 by Shane Saxon - contact@saxondigital.net
*
*  Please see main.c for a complete list of additional code contributors.
*
*  To the extent possible under law, the author(s) have dedicated all copyright 
*  and related and neighboring rights to this software to the public domain
*  worldwide. This software is distributed without any warranty.
*
*  Released under the CC0 license, which is GPL compatible.
*
*  You should have received a copy of the CC0 Public Domain Dedication along
*  with this software (license file named COPYING.txt). If not, see
*  http://creativecommons.org/publicdomain/zero/1.0/
*
* --------------------------------------------------------------------------- */

#include "npcsv.h"


#include "../../os/npos.h"
#include "../npfile.h"


//-----------------------------------------------------------------------------
void npInitCSV (void* dataRef)
{
	return;
}

//-----------------------------------------------------------------------------
void npCloseCSV (void* dataRef)
{
	return;
}

//-----------------------------------------------------------------------------
// mode usage: 3 parameters "r/w/a" followed by optional "b" and optional "+"
// example mode = "rb+"
// r open file for reading
// w open file for overwriting. Create new file if doesn't already exist
// a open file for appending data. 
// b open file as a binary file
// + open file for read and writing
//-----------------------------------------------------------------------------
FILE* npOpenCSV (const char* fileName, const char* mode, void* dataRef)
{
	FILE* file = NULL;

	file = fopen(fileName, mode); 

	if ( file == NULL )
	{
		perror("Failed to open/create file");
		return NULL;
	}

	// add code to add the file ptr to the data->map							debug, zz

	return file;
}


//-----------------------------------------------------------------------------
int npReadCSV (void* readBuffer, int elementSize, int elementCount, 
				 FILE* file, void* dataRef)
{
	int bytes = 0;

	bytes = fread (readBuffer, elementSize, elementCount, file);
	
	if ( bytes == 0 )
		printf("Failed Reading\n");
	else
		printf("Success Reading\n");

	return bytes;
}


//-----------------------------------------------------------------------------
void npTestCSV (void* fileName, int wordSize, int size, void* dataRef)
{
	FILE *file;
	size_t count;
	char *str = (char*)fileName;
	int i = 0;


	char buffer[256];
/*
	// get the Current Working Directory
	npGetCWD (buffer, &size);

	printf("CWD path : ");
	for(i = 0; i < size; i++)
		printf ("%c", buffer[i]);

	printf("\n");
*/
	// open/create and write to the file, "w" overwrites existing files
	file = npOpenCSV (fileName, "r+", dataRef);

	// write the file path in the file as text
	count = npWriteCSV (buffer, 1, strlen(buffer), file, dataRef);

	printf("wrote %d bytes.\n", count); 

	// read the file
//	npFileRewind(file);

	size = npReadCSV (buffer, 1, 256, file, dataRef);


	printf("file contents : \n");
	for(i = 0; i < size; i++)
		printf ("%c", buffer[i]);

//mkdir(strB, S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);

}

//-----------------------------------------------------------------------------
FILE* npCSVDialog (const char* fileName, int dialogType, void* dataRef)
{	
	
	switch (dialogType)
	{
		case kNPfileDialogNew : ; break;

		case kNPfileDialogOpen : ; break;
		case kNPfileDialogClose : ; break;

		case kNPfileDialogSave : ; break;
		case kNPfileDialogSaveAs : ; break;

		case kNPfileDialogImport : ; break;
		case kNPfileDialogExport : ; break;

		default : break;
	}

	return NULL;
}


//-----------------------------------------------------------------------------
int npWriteCSV (const void* str, int wordSize, int size, FILE* file, void* dataRef)
{
	return fwrite( str, 1, strlen(str), file );
}



void npSaveCSV (FILE* file, void* dataRef)
{
	return;
}


//-----------------------------------------------------------------------------
void npImportCSV (FILE* file, int type, void* dataRef)
{
	return;
}

//-----------------------------------------------------------------------------
void npExportCSV (FILE* file, void* dataRef)
{
	return;
}

//-----------------------------------------------------------------------------

// if not saved then opens SaveAs dialog before reseting app
void npCloseFileCSV (int size, void* dataRef)
{
	return;
}

//-----------------------------------------------------------------------------
void npCSVGetAttrib (int size, void* dataRef)
{
	return;
}

//-----------------------------------------------------------------------------
void npCSVSetAttrib (int size, void* dataRef)
{
	return;
}

//-----------------------------------------------------------------------------
//passed an open State File and reads it in based on type
void npCSVtoMap (FILE* file, int type, void*dataRef)
{
	size_t count = 0;
	int i = 0;
	int byteCount = 0;
	char* bufferIndex = 0;
	
	pData data = (pData) dataRef;
	pNPnode node = NULL;


	char* buffer = (char*)malloc(kNPmapFileBufferMax);
	if (buffer == NULL)
	{
		printf ("err 4285 - malloc failed, cannot write file");
		return;
	}

	if( file != NULL )   // check to see if file exists
	{
		printf("Reading File...\n");
		count = npFileRead (buffer, 1, kNPmapFileBufferMax, file, dataRef);
		printf("Bytes Read: %d\n", count);

		npFileClose (file, dataRef);	//done reading, close file
		
		if (count > 0)
		{
			printf("Loading File\n");			//add table type handling, debug zz
			npLoadMapFile (buffer, 1, count, dataRef);	//process data
			printf ("Done\n\n");
		}
		else
			printf("err 4297 - zero bytes read\n");
	}
	else
		printf("err 4298 - cannot open file \n");


	free (buffer);

	return;
}
/*
enum
{
	kNPtableGlobals,			//global settings
	kNPtableNode,				//entire node scene state
	kNPtableTag,				//node record_id based text tags
	kNPtableChMap,				//maps nodes to tracks
	kNPtableTrack,				//raw channel track data
	kNPtableOSC,				//maps OSC commands and node data
	kNPtableKeyMap,				//maps keyboard functions
	kNPtableCount			
};
*/

//zz debug, need a table similar to npmap.c generates for node header with ptrs like chmap
//need arrays for sub elements to, color = black, white...

//-----------------------------------------------------------------------------
char* npGetValueStr(char* dest, pNPmapType2 item, void* dataRef) //*(int*)mapGlobals[id].mapPtr,
{
	//*(int*)mapGlobals[id].mapPtr,

	
	pNPubyteRGBA ubyteRGBA = NULL;
	pNPfloatRGBA floatRGBA = NULL;
	pNPgl gl = NULL;

	//	_itoa(*(int*)item->mapPtr, dest, 0);	//zz debug optimize

	switch (item->type)
	{
		case kNPint : sprintf (dest, "%d", *(int*)item->mapPtr); break;
		case kNPfloat : sprintf(dest, "%f", *(float*)item->mapPtr); break;
		case kNPcstrPtr : sprintf (dest, "%s", (char*)item->mapPtr); break;

		case kNPubyteRGBA :
			ubyteRGBA = item->mapPtr;
			sprintf(dest, "%d,%d,%d,%d", 
					ubyteRGBA->r,
					ubyteRGBA->g,
					ubyteRGBA->b,
					ubyteRGBA->a
					);
			break;

		case kNPfloatRGBA :
			floatRGBA = item->mapPtr;
			sprintf(dest, "%f,%f,%f,%f",
					floatRGBA->r,
					floatRGBA->g,
					floatRGBA->b,
					floatRGBA->a
					);
			break;

		case kNPgWindowPosition :
			gl = item->mapPtr;
			sprintf(dest, "%d,%d,%d,%d,%d",
					gl->windowID,		//zz debug
					gl->position.x,
					gl->position.y,
					gl->windowSize.x,
					gl->windowSize.y
					);
			break;

		default : //report err
				strcpy (dest, "22");//"err 9999 - unknown pNPmapType item");
			break;
	}

	
	return  dest;
}


//-----------------------------------------------------------------------------

	// ISSUES 
	// best not to limit osc str length
	// need to get global ptr when populating list, or servicing it....
	// advantage to both runtime const to ptr conversion and pre-mapped
	// may change data pointer for struct member mapping pointer offsets
	// nodeMemberPtrOffset = (&node->translate - node)
	// (pNPfloatXYZ)node+nodeMemberPtrOffset

	// for example /np/node/42 iiiifff...i 42 1 1 7... 420
	// needs to get node 42 then populate, more of an offset method


void* nMapIDtoPtr (int mapID, void* dataRef);
//-----------------------------------------------------------------------------
void* nMapIDtoPtr (int mapID, void* dataRef)
{
	int i = 0;
	int itemCount = 0;

	pData data = (pData) dataRef;

	pNPmapType2 mapType = NULL;


	//get the map and itemCount
	//first check globals
	mapType = data->map.typeMapGlobals;
	itemCount = data->map.globalsCount;

	//scan map using the ID and if found return a pointer to the data
	for (i=0; i <= itemCount; i++)
	{
		if (mapType[i].ptrID == mapID)
			return mapType[i].mapPtr;
	}

	//then check osc
	mapType = data->map.typeMapOSC;
	itemCount = data->map.oscCount;

	//scan map using the ID and if found return a pointer to the data
	for (i=0; i <= itemCount; i++)
	{
		if (mapType[i].ptrID == mapID)
			return mapType[i].mapPtr;
	}

	return NULL;
}


char* npGetMapTypeName (int mapID, void* dataRef);
//-----------------------------------------------------------------------------
char* npGetMapTypeName (int mapID, void* dataRef)
{
	int i = 0;
	int itemCount = 0;

	pData data = (pData) dataRef;

	pNPmapType2 mapType = NULL;


	//get the map and itemCount
	//first check globals
	mapType = data->map.typeMapGlobals;
	itemCount = data->map.globalsCount;

	//scan map using the ID and if found return a pointer to the data
	for (i=0; i <= itemCount; i++)
	{
		if (mapType[i].ptrID == mapID)
			return mapType[i].attribute;
	}

	//then check osc
	mapType = data->map.typeMapOSC;
	itemCount = data->map.oscCount;

	//scan map using the ID and if found return a pointer to the data
	for (i=0; i <= itemCount; i++)
	{
		if (mapType[i].ptrID == mapID)
			return mapType[i].attribute;
	}

	return NULL;
}


int npMapStrToID (const char* str, void* dataRef);
//-----------------------------------------------------------------------------
int npMapStrToID (const char* str, void* dataRef)
{
	int i = 0;
	int result = 0;
	int itemCount = 0;

	pData data = (pData) dataRef;

	pNPmapType2 mapType = NULL;
	pNPmapType2 mapItem = NULL;

	//first check globals
	mapType = data->map.typeMapGlobals;
	itemCount = data->map.globalsCount;

	//scan attribute lists and return the corresponding constant kNP... = 42
	for (i=0; i <= itemCount; i++)
	{
		mapItem = &mapType[i];

		if (strncmp(str, (const char*)mapItem->attribute, kNPmapTypeTagMax) == 0)
		{
			result = mapItem->ptrID;
			break;
		}
	}
		
	//then check OSC map
	mapType = data->map.typeMapOSC;
	itemCount = data->map.oscCount;

	//scan attribute lists and return the corresponding constant kNP... = 42
	for (i=0; i <= itemCount; i++)
	{
		mapItem = &mapType[i];

		if (strncmp(str, (const char*)mapItem->attribute, kNPmapTypeTagMax) == 0)
		{
			result = mapItem->ptrID;
			break;
		}
	}

	return result;
}

void* npMapStrToPtr (const char* str, void* dataRef);
//-----------------------------------------------------------------------------
void* npMapStrToPtr (const char* str, void* dataRef)
{
	//quick implementation... not efficient, make direct like others..
//	return nMapIDtoPtr ( npMapStrToID (str, dataRef), dataRef );

	int i = 0;
	int itemCount = 0;

	pData data = (pData) dataRef;

	pNPmapType2 mapType = NULL;

	//get the map and itemCount
	mapType = data->map.typeMapGlobals;
	itemCount = data->map.globalsCount;

	//scan map using the ID and if found return a pointer to the data
	for (i=0; i <= itemCount; i++)
	{
		if (strncmp(str, (const char*)mapType[i].attribute, kNPmapTypeTagMax) == 0)
			return mapType[i].mapPtr;
	}

	//get the map and itemCount
	mapType = data->map.typeMapOSC;
	itemCount = data->map.oscCount;

	//scan map using the ID and if found return a pointer to the data
	for (i=0; i <= itemCount; i++)
	{
		if (strncmp(str, (const char*)mapType[i].attribute, kNPmapTypeTagMax) == 0)
			return mapType[i].mapPtr;
	}

	return NULL;
}


//-----------------------------------------------------------------------------
void* npMapAddressToPtr (const char* str, void* dataRef)
{
	//quick implementation... not efficient, make direct like others..
//	return nMapIDtoPtr ( npMapStrToID (str, dataRef), dataRef );

	int i = 0;
	int itemCount = 0;

	pData data = (pData) dataRef;

	pNPmapType2 mapType = NULL;

	//get the map and itemCount
	mapType = data->map.typeMapGlobals;
	itemCount = data->map.globalsCount;

	//scan map using the ID and if found return a pointer to the data
	for (i=0; i <= itemCount; i++)
	{
		if (strncmp(str, (const char*)mapType[i].address, kNPmapTypeTagMax) == 0)
			return mapType[i].mapPtr;
	}

	//get the map and itemCount
	mapType = data->map.typeMapOSC;
	itemCount = data->map.oscCount;

	//scan map using the ID and if found return a pointer to the data
	for (i=0; i <= itemCount; i++)
	{
		if (strncmp(str, (const char*)mapType[i].address, kNPmapTypeTagMax) == 0)
			return mapType[i].mapPtr;
	}

	return NULL;
}


//-----------------------------------------------------------------------------
pNPmapType2 npMapAddressToMapTypeItem (const char* str, void* dataRef)
{
	//quick implementation... not efficient, make direct like others..
//	return nMapIDtoPtr ( npMapStrToID (str, dataRef), dataRef );

	int i = 0;
	int itemCount = 0;

	pData data = (pData) dataRef;

	pNPmapType2 mapType = NULL;

	//get the map and itemCount
	mapType = data->map.typeMapGlobals;
	itemCount = data->map.globalsCount;

	//scan map using the ID and if found return a pointer to the data
	for (i=0; i <= itemCount; i++)
	{
		if (strncmp(str, (const char*)mapType[i].address, kNPmapTypeTagMax) == 0)
			return &mapType[i];
	}

	//get the map and itemCount
	mapType = data->map.typeMapOSC;
	itemCount = data->map.oscCount;

	//scan map using the ID and if found return a pointer to the data
	for (i=0; i <= itemCount; i++)
	{
		if (strncmp(str, (const char*)mapType[i].address, kNPmapTypeTagMax) == 0)
			return &mapType[i];
	}

	return NULL;
}


//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void npMapTypeInit (void* dataRef)
{
	int i = 0;
	int size = 0;

	pData data = (pData) dataRef;

	//build descriptor table for globals with pointer to specific elements
	//note that mapItemID = attribute = address (CSV field name for native types)
	NPmapType2 mapGlobals[] = {

		{ (void*)NULL,				kNPgNull,			kNPint,				"globals",	0,	"null",				"csv", "null", "NULL map_type item" },
		{ &data->io.gl.alphaMode,	kNPgAlphaMode,		kNPint,				"globals",	0,	"alpha_mode",		"csv", "alpha_mode", "Transparency Mode"	 },
		{ &data->io.gl.fullscreen,	kNPgFullscreen,		kNPint,				"globals",	0,	"full_screen",		"csv", "full_screen", "Fullscreen Mode" },

		{ &data->io.clear,			kNPgBackgroundRGBA,	kNPfloatRGBA,		"globals",	0,	"background_rgba",	"csv", "background_rgba", "Background color RGBA" },
		{ &data->io.clear.r,		kNPgBackgroundR,	kNPfloat,			"globals",	0,	"background_r",		"csv", "background_r", "Background color Red" },
		{ &data->io.clear.g,		kNPgBackgroundG,	kNPfloat,			"globals",	0,	"background_g",		"csv", "background_g", "Background color Green" },
		{ &data->io.clear.b,		kNPgBackgroundB,	kNPfloat,			"globals",	0,	"background_b",		"csv", "background_b", "Background color Blue" },
		{ &data->io.clear.a,		kNPgBackgroundA,	kNPfloat,			"globals",	0,	"background_a",		"csv", "background_a", "Background color Alpha" },
		
		{ &data->io.gl,				kNPgWindowPosition,	kNPgWindowPosition,	"globals",	0,	"window_position",	"csv", "window_position", "window_id, x, y, width, height" },
		{ &data->io.url,			kNPgBrowserURL,		kNPcstrPtr,			"globals",	0,	"browser_url",		"csv", "browser_url", "Browser URL path used for querying by record_id" },

		{ &data->map.globalsCount,	kNPitemCount,		kNPint,				"globals",	0,	"item_count",		"csv", "item_count", "Last row is item count" },
	};

//-------------------------------------- repeat above for mapOSC
//JJ-zz-osc

	//may add connection specific OSC maps for greater 3rd party support //zz debug

	//zz-JJ planning to add base_type(globals,node...) and record_id columns

	NPmapType2 mapOSC[] = {

		{ (void*)NULL,				kNPgNull,			kNPint,				"globals",	0,	"null",				"osc", "/np/null", "i" },
		{ &data->io.gl.alphaMode,	kNPgAlphaMode,		kNPint,				"globals",	0,	"alpha_mode",		"osc", "/np/gl/1/alpha_mode", "i" },
		{ &data->io.gl.fullscreen,	kNPgFullscreen,		kNPint,				"globals",	0,	"full_screen",		"osc", "/np/gl/1/fullscreen", "i" },

		{ &data->io.clear,			kNPgBackgroundRGBA,	kNPfloatRGBA,		"globals",	0,	"background_rgba",	"osc", "/np/gl/1/background_rgba", "ffff" },
		{ &data->io.clear.r,		kNPgBackgroundR,	kNPfloat,			"globals",	0,	"background_r",		"osc", "/np/gl/1/background_r", "f" },
		{ &data->io.clear.g,		kNPgBackgroundG,	kNPfloat,			"globals",	0,	"background_g",		"osc", "/np/gl/1/background_g", "f" },
		{ &data->io.clear.b,		kNPgBackgroundB,	kNPfloat,			"globals",	0,	"background_b",		"osc", "/np/gl/1/background_b", "f" },
		{ &data->io.clear.a,		kNPgBackgroundA,	kNPfloat,			"globals",	0,	"background_a",		"osc", "/np/gl/1/background_a", "f" },
		
		{ &data->io.gl,				kNPgWindowPosition,	kNPgWindowPosition,	"globals",	0,	"window_position",	"osc", "/np/gl/1/window_position", "iiiii" },
		{ &data->io.url,			kNPgBrowserURL,		kNPcstrPtr,			"globals",	0,	"browser_url",		"osc", "/np/io/browser_url", "s" },
	
		// above is a copy of the mapGlobals... 
		//zz - update to auto-generate global OSC addresses and type tags, dynamic node parsing...

		// below is 3rd party OSC schemas
		// REPEAT ptr ID allowed, first OSC address will be used when sending
		{ &data->io.url,			kNPgBrowserURL,			kNPcstrPtr,		"globals",	0,	"browser_url",		"osc", "/1/text", "s" },

		{ &data->io.clear.r,		kNPgBackgroundR,		kNPfloat,		"globals",	0,	"background_r",		"osc", "/1/fader1", "f" },
		{ &data->io.clear.g,		kNPgBackgroundG,		kNPfloat,		"globals",	0,	"background_g",		"osc", "/1/fader2", "f" },
		{ &data->io.clear.b,		kNPgBackgroundB,		kNPfloat,		"globals",	0,	"background_b",		"osc", "/1/fader3", "f" },
		{ &data->io.clear.a,		kNPgBackgroundA,		kNPfloat,		"globals",	0,	"background_a",		"osc", "/1/fader4", "f" },
		
		{ &data->map.oscCount,		kNPitemCount,			kNPint,			"globals",	0,	"item_count",		"osc", "/np/map/osc/item_count", "i" }
	};

	// MEMORY for mapGlobals
	// copy the structure to newly allocated RAM, proably a better way, zz
	size = sizeof(mapGlobals);
	data->map.typeMapGlobals = npMalloc(0, size, data);

	memcpy(data->map.typeMapGlobals, (const void*)mapGlobals, size);

	//scan the list looking for kNPcount to dynamically get the size
	i = 0;
	while (mapGlobals[i++].ptrID != kNPitemCount) data->map.globalsCount++;
	

	// REPEAT for mapOSC
	// copy the structure to newly allocated RAM, proably a better way, zz
	size = sizeof(mapOSC);
	data->map.typeMapOSC = npMalloc(0, size, data);

	memcpy(data->map.typeMapOSC, (const void*)mapOSC, size);

	//scan the list looking for kNPcount to dynamically get the size
	i = 0;
	while (mapOSC[i++].ptrID != kNPitemCount) data->map.oscCount++;


	//-------------
	// TEST MAP		//zz debug
//	printf("\n TEST GLOBALS alphaMode: %d\n\n", *(int*)mapGlobals[1].mapPtr);
	printf("\n TEST MAP kNPitemCount: %d\n", data->map.globalsCount);
	printf(" TEST MAP size: %d\n", sizeof(mapGlobals));
	printf(" TEST MAP id: %d\n", npMapStrToID("full_screen\0",data));
	printf(" TEST MAP value: %d\n\n", *(int*)npMapStrToPtr ("full_screen\0", data));	

//	printf("\n TEST GLOBALS alphaMode: %d\n\n", *(int*)mapGlobals[1].mapPtr);
	printf("\n TEST MAP kNPitemCount: %d\n", data->map.oscCount);
	printf(" TEST MAP size: %d\n", sizeof(mapOSC));
	printf(" TEST MAP id: %d\n", npMapAddressToPtr("/1/fader4\0",data));
	printf(" TEST MAP value: %f\n\n", *(float*)npMapAddressToPtr ("/1/fader4\0", data));

	//this should change the background color 'b' blue component
//	*(float*)npMapAddressToPtr ("/1/fader1\0", data) = 0.6f;
//	*(float*)npMapAddressToPtr ("/1/fader2\0", data) = 0.4f;
//	*(float*)npMapAddressToPtr ("/1/fader3\0", data) = 0.6f;

	*(float*)npMapAddressToMapTypeItem ("/1/fader1\0", data)->mapPtr = 0.6f;
	*(float*)npMapAddressToMapTypeItem ("/1/fader2\0", data)->mapPtr = 0.4f;
	*(float*)npMapAddressToMapTypeItem ("/1/fader3\0", data)->mapPtr = 0.6f;
}

//-----------------------------------------------------------------------------
int npMapToCSV (char* csvStr, int mapType, int size, void* dataRef)
{
	int id = 0;
	int itemCount = 0;

	int n = 0;				//curs byte count offset
	char* curs = csvStr;

	char str[4096];			//temp memory for value string, optimize this out, //zz debug			


	pData data = (pData) dataRef;

	pNPmapType2 mapTypePtr = NULL;
	pNPmapType2 mapItem = NULL;


//zz-JJ		itemCount = data->map.globalsCount;

	if ( mapType == kNPtableMapOSC )
	{
		mapTypePtr = data->map.typeMapOSC;
		itemCount = data->map.oscCount;

		
		// write the first row header, includes CRLF line ending
		// ---
		// 'item_id' = record_id for nodes or window_id for GL window context... etc.
		//
		// 'map_id' = maps the data location... file, db table, list ptr
		// pathMap connects MySQL, URL, OSC, files...
		// table names such as 'globals', 'node', 'gl', 'track', 'cmd'
		// custom table names... node_tbl_5... or MyCustomTableName...
		//example: 1,node,42,translate_xyz,/np/node/42/translate_xyz
		// ---
		n += sprintf (curs, "np_oscmap_id,path_id,item_id,attribute,format,address_data,tag_desc,\n");
		
		for (id = 1; id <= itemCount; id++)
		{
			mapItem = &mapTypePtr[id];

			n += sprintf ((curs + n), "%d,%s,%d,%s,%s,\"%s\",\"%s\"\n",	//zz optimize with npitoa...
						id,
						mapItem->path, //"node"	//mapItem->baseType	//zz-JJ this will be added to array. mapItem->baseType
						mapItem->id,		//list index or 0 for single element
						mapItem->attribute,
						mapItem->format,
						mapItem->address,//npGetValueStr(str, mapItem, data),	//format value str
						mapItem->tag
					 );
		}
	}
	else if ( mapType == kNPtableGlobals )
	{
	//		npMapTypeInit(data);
		mapTypePtr = data->map.typeMapGlobals;
		itemCount = data->map.globalsCount;

		//write the first row header, includes CRLF line ending
		n += sprintf (curs, "np_globals_id,path_id,item_id,attribute,format,address_data,tag_desc\n");
		//example: 1,gl,1,clear_rgba,"1.0,0.5,0.5,1.0","background color RGBA"
	
	//zz-JJ		n += sprintf (curs, "np_global_id,attribute,value,desc\n");

		//loop and write out all globals in the mapType[] list
		//populates the rows using the globals map descriptor array 
		for (id = 1; id <= itemCount; id++)
		{
	//zz-JJ		mapItem = &data->map.typeMapGlobals[id];
			mapItem = &mapTypePtr[id];

			n += sprintf ((curs + n), "%d,%s,%d,%s,%s,\"%s\",\"%s\"\n",	//zz optimize with npitoa...
			/*			id,
						"globals", //"node"	//mapItem->baseType	//zz-JJ this will be added to array. mapItem->baseType
						0,				//mapItem->recordID
						mapItem->name,
						"csv",
						npGetValueStr(str, mapItem, data),	//format value str
						mapItem->tag

			*/			id,
						mapItem->path, //"node"	//mapItem->baseType	//zz-JJ this will be added to array. mapItem->baseType
						mapItem->id,		//list index or 0 for single element
						mapItem->attribute,
						mapItem->format,
						npGetValueStr(str, mapItem, data),	//format value str
						mapItem->tag
					 );
		}
	}

	return n;
}


//-----------------------------------------------------------------------------
int npOpenMapCSV (char* filePath, int mapType, void* dataRef)
{
	int result = 0;
	int eof = 0;

	char* curs = NULL;	//buffer

	FILE* file = NULL;
	size_t count = 0;
	int i = 0;
	int byteCount = 0;
	char* bufferIndex = 0;
	char* buffer = NULL;

	pData data = (pData) dataRef;

	curs = buffer = npMalloc(0,kNPfileBlockSize,dataRef);
		if (!buffer) return 0;

	printf("\nOpen File: %s\n", filePath);
	file = npFileOpen (filePath, "r", dataRef);	// open file in read mode "r"

	if( file != NULL )   // check to see if file exists
	{	
		buffer = (char*) malloc(kNPmapFileBufferMax);
		if (buffer == NULL)
		{
			printf ("err 4285 - malloc failed, cannot write file");
			return 0;
		}

		printf("Reading File... ");
		count = npFileRead (buffer, 1, kNPmapFileBufferMax, file, dataRef);
		printf("Bytes Read: %d ", count);

		npFileClose (file, dataRef);	//done reading, close file
		
		if (count > 0)
		{
			printf("converting\n");
	//		npLoadMapFile (buffer, wordSize, count, dataRef);	//process data




	//write the first row header, includes CRLF line ending
	//id the table using the first field of the header row
	if (strncmp(curs, "np_global_id", 12) != 0)
	{
		npPostMsg("err 7822 - np_global_id csv header error", kNPmsgErr, data);
	}

	count += i = npSeekToNextLine( curs ); curs = &curs[i];	//skip blank line


	//loop and write out all globals in the mapType[] list
	//populates the rows using the globals map descriptor array 
/*	for (id = 1; id <= itemCount; id++)
	{
		mapItem = &data->map.typeMapGlobals[id];

		n += sprintf ((curs + n), "%d,%s,\"%s\",\"%s\"\n",	//zz optimize with npitoa...
					id,
	//				type?
					mapItem->name,
					mapItem->tag,
					npGetValueStr(str, mapItem, data)	//format value str			
				 );
	}
*/

	// 
	//similar to npCSVtoTag()
	//npLoadMapFile()

	//get the first row
	while (!eof)
	{
		//goto start of next row

		//id = npstrtoi(&cursor);
		//type = kNPgFullscreen
		//name =
		//tag =	
		//valueStr =

		eof = true;
	}

				printf ("Done\n");
		}
		else
			printf("err 4287 - zero bytes read\n");

		free (buffer);
	}
	else
		printf("err 4288 - cannot open file \n");


	return result;
}

/*
//-----------------------------------------------------------------------------
int npSaveGlobalsCSV (char* fileName, void* dataRef)
{
	char* path = NULL;
	FILE* fileRef = NULL;

	pData data = (pData) dataRef;

	csvStr = npMalloc (kNPfileBlock, kNPfileBlockSize, data);

	//if no fileName then assign default time-stamped name
	if(!fileName)
	{
		path = npMalloc(0, kNPmaxPath, data);
		strcpy (path, "antzglobals.csv");			//add time stamp and CWD	//zz debug
	}
	else
		path = fileName;

	//open the file for writing
	fileRef = npFileOpen ((const char*)path, "rw+", data);


	//get the first row header, includes CRLF line ending
	size = npGetGlobalsCSV (csvStr, data);

	npFileWrite ((const void*)csvStr, 1, size, fileRef, data);
	
	npFree(path);
	npFree(csvStr);

	return size;
}
*/


//-----------------------------------------------------------------------------
int npOpenGlobalsCSV (char* filePath, int wordSize, int size, void* dataRef)
{
	return npOpenMapCSV(filePath, kNPtableGlobals, dataRef);
}

//
//-----------------------------------------------------------------------------
int npSaveMapToCSV (char* filePath, int mapType, void* dataRef)
{
	int i = 0;
	int err = 0;
	int size = 0;
	int total = 0;
	int rootIndex = 0;
	size_t count = 0;
	FILE* file = NULL;

	pData data = (pData) dataRef;
	pNPnode node = NULL;

	char* buffer = (char*) malloc(kNPfileBlockSize);
	if (buffer == NULL)
	{
		printf ("4214 error - malloc failed, cannot write file\n");
		free (buffer);
		return 0;
	}

	// open the file, "w+" overwrites existing files
	printf ("\nSave File: %s\n", filePath);
	file = npFileOpen (filePath, "w+", dataRef);

	if (file == NULL)
	{
		printf("err 4215 - file open failed, cannot write file\n");							//debug, add err reporting, zz
		free (buffer);
		return 0;
	}
	
	// copies current globals to the write buffer, formats as CSV string
	printf("Processing...\n");
	count = npMapToCSV (buffer, mapType, kNPfileBlockSize, dataRef);

	printf("Writing...\n");
	count = npFileWrite (buffer, 1, count, file, dataRef);


	// print first line of file contents
	if (count > 0)
	{
		printf("Bytes Written: %d\n", count);
		npFileRewind(file);
		size = npFileRead (buffer, 1, 79, file, dataRef);
		printf("File Contents:\n");
		for( i = 0; i < size; i++ )
			printf("%c", buffer[i]);
		printf("\n");
	}
	else
		printf("err 4216 - file write failure, zero bytes written\n");

	err = npFileClose(file, dataRef);
	if (err)
		printf("err 4217 - file close failed, return: %d\n", i);

	npPostMsg("Done writing Globals to CSV", kNPmsgCtrl, data);

	free (buffer);

	return count;
}
